"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractStaticName = exports.extractStaticString = exports.getCallType = exports.prettyNodeLocation = exports.CallType = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@yarnpkg/core");
const ts = tslib_1.__importStar(require("typescript"));
var CallType;
(function (CallType) {
    CallType[CallType["REQUIRE"] = 0] = "REQUIRE";
    CallType[CallType["RESOLVE"] = 1] = "RESOLVE";
    CallType[CallType["IMPORT"] = 2] = "IMPORT";
})(CallType = exports.CallType || (exports.CallType = {}));
function prettyNodeLocation(configuration, node) {
    const { fileName } = node.getSourceFile();
    const { line, character } = ts.getLineAndCharacterOfPosition(node.getSourceFile(), node.getStart());
    return [
        `${core_1.formatUtils.pretty(configuration, fileName, core_1.formatUtils.Type.PATH)}`,
        `${core_1.formatUtils.pretty(configuration, line + 1, core_1.formatUtils.Type.NUMBER)}`,
        `${core_1.formatUtils.pretty(configuration, character + 1, core_1.formatUtils.Type.NUMBER)}`,
    ].join(`:`);
}
exports.prettyNodeLocation = prettyNodeLocation;
function getCallType(call) {
    switch (call.expression.kind) {
        // require(...)
        case ts.SyntaxKind.Identifier:
            {
                const ident = call.expression;
                if (ident.text !== `require`)
                    return null;
                return CallType.REQUIRE;
            }
            break;
        // require.resolve(...)
        case ts.SyntaxKind.PropertyAccessExpression:
            {
                const access = call.expression;
                if (!ts.isIdentifier(access.expression))
                    return null;
                const ident = access.expression;
                if (ident.text !== `require`)
                    return null;
                if (access.name.text !== `resolve`)
                    return null;
                return CallType.RESOLVE;
            }
            break;
        // import(...)
        case ts.SyntaxKind.ImportKeyword:
            {
                return CallType.IMPORT;
            }
            break;
        default:
            {
                return null;
            }
            break;
    }
}
exports.getCallType = getCallType;
function extractStaticString(node) {
    switch (node.kind) {
        case ts.SyntaxKind.NoSubstitutionTemplateLiteral:
            {
                const literal = node;
                return literal.text;
            }
            break;
        case ts.SyntaxKind.StringLiteral:
            {
                const literal = node;
                return literal.text;
            }
            break;
        default:
            {
                return null;
            }
            break;
    }
}
exports.extractStaticString = extractStaticString;
function extractStaticName(node) {
    switch (node.kind) {
        case ts.SyntaxKind.Identifier:
            {
                const identifier = node;
                return identifier.text;
            }
            break;
        case ts.SyntaxKind.StringLiteral:
            {
                const literal = node;
                return literal.text;
            }
            break;
        default:
            {
                return null;
            }
            break;
    }
}
exports.extractStaticName = extractStaticName;
