"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeRepoUrl = void 0;
const hosted_git_info_parse_1 = require("../hosted-git-info-parse");
function normalizeRepoUrl(url, { git = false } = {}) {
    // "git+https://" isn't an actual Git protocol. It's just a way to
    // disambiguate that this URL points to a Git repository.
    url = url.replace(/^git\+https:/, `https:`);
    // We support this as an alias to GitHub repositories
    url = url.replace(/^(?:github:|https:\/\/github\.com\/|git:\/\/github\.com\/)?(?!\.{1,2}\/)([a-zA-Z0-9._-]+)\/(?!\.{1,2}(?:#|$))([a-zA-Z0-9._-]+?)(?:\.git)?(#.*)?$/, `https://github.com/$1/$2.git$3`);
    // We support GitHub `/tarball/` URLs
    url = url.replace(/^https:\/\/github\.com\/(?!\.{1,2}\/)([a-zA-Z0-9._-]+)\/(?!\.{1,2}(?:#|$))([a-zA-Z0-9._-]+?)\/tarball\/(.+)?$/, `https://github.com/$1/$2.git#$3`);
    if (git) {
        // Try to normalize the URL in a way that git accepts.
        const parsedUrl = (0, hosted_git_info_parse_1.tryParseGitURL)(url);
        if (parsedUrl)
            url = parsedUrl.href;
        // The `git+` prefix doesn't mean anything at all for Git.
        url = url.replace(/^git\+([^:]+):/, `$1:`);
    }
    return url;
}
exports.normalizeRepoUrl = normalizeRepoUrl;
