import { Cache, Locator, Descriptor, Ident, Project, FetchOptions, Package } from '@yarnpkg/core';
import { PortablePath } from '@yarnpkg/fslib';
import { parsePatchFile } from './tools/parse';
export { applyPatchFile } from './tools/apply';
export { parsePatchFile };
export declare function isPatchDescriptor(descriptor: Descriptor): boolean;
export declare function isPatchLocator(locator: Locator): boolean;
export declare function parseDescriptor(descriptor: Descriptor): {
    sourceDescriptor: Descriptor;
    parentLocator: Locator | null;
    patchPaths: PortablePath[];
    sourceVersion: string | null;
};
export declare function parseLocator(locator: Locator): {
    sourceLocator: Locator;
    parentLocator: Locator | null;
    patchPaths: PortablePath[];
    sourceVersion: string | null;
};
export declare function unpatchDescriptor(descriptor: Descriptor): Descriptor;
export declare function unpatchLocator(locator: Locator): Locator;
export declare function ensureUnpatchedDescriptor(descriptor: Descriptor): Descriptor;
export declare function ensureUnpatchedLocator(locator: Locator): Locator;
export declare function makeDescriptor(ident: Ident, { parentLocator, sourceDescriptor, patchPaths }: Pick<ReturnType<typeof parseDescriptor>, `parentLocator` | `sourceDescriptor` | `patchPaths`>): Descriptor;
export declare function makeLocator(ident: Ident, { parentLocator, sourcePackage, patchPaths, patchHash }: Omit<ReturnType<typeof parseLocator>, 'sourceLocator' | 'sourceVersion'> & {
    sourcePackage: Package;
    patchHash: string;
}): Locator;
export declare function extractPatchFlags(patchPath: PortablePath): {
    optional: boolean;
};
export declare function isParentRequired(patchPath: PortablePath): boolean;
export declare function loadPatchFiles(parentLocator: Locator | null, patchPaths: Array<PortablePath>, opts: FetchOptions): Promise<{
    source: string | null;
    optional: boolean;
}[]>;
export declare function extractPackageToDisk(locator: Locator, { cache, project }: {
    cache: Cache;
    project: Project;
}): Promise<PortablePath>;
export declare function diffFolders(folderA: PortablePath, folderB: PortablePath): Promise<string>;
export declare function makePatchHash(patchFiles: Array<{
    source: string | null;
    optional: boolean;
}>, sourceVersion: string | null): string;
