import { Descriptor, LocatorHash, InstallPackageExtraApi } from '@yarnpkg/core';
import { FetchResult, Locator, Package } from '@yarnpkg/core';
import { Linker, LinkOptions, MinimalLinkOptions, Installer } from '@yarnpkg/core';
import { PortablePath } from '@yarnpkg/fslib';
import { PnpSettings } from '@yarnpkg/pnp';
export declare class PnpLinker implements Linker {
    protected mode: string;
    private pnpCache;
    getCustomDataKey(): string;
    supportsPackage(pkg: Package, opts: MinimalLinkOptions): boolean;
    findPackageLocation(locator: Locator, opts: LinkOptions): Promise<PortablePath>;
    findPackageLocator(location: PortablePath, opts: LinkOptions): Promise<Locator | null>;
    makeInstaller(opts: LinkOptions): PnpInstaller;
    private isEnabled;
}
export declare class PnpInstaller implements Installer {
    protected opts: LinkOptions;
    protected mode: string;
    private readonly asyncActions;
    private readonly packageRegistry;
    private readonly virtualTemplates;
    private isESMLoaderRequired;
    constructor(opts: LinkOptions);
    private customData;
    attachCustomData(customData: any): void;
    installPackage(pkg: Package, fetchResult: FetchResult, api: InstallPackageExtraApi): Promise<{
        packageLocation: PortablePath;
        buildRequest: import("@yarnpkg/core").BuildRequest | null;
    }>;
    attachInternalDependencies(locator: Locator, dependencies: Array<[Descriptor, Locator]>): Promise<void>;
    attachExternalDependents(locator: Locator, dependentPaths: Array<PortablePath>): Promise<void>;
    finalizeInstall(): Promise<{
        customData: {
            store: Map<LocatorHash, {
                manifest: {
                    scripts: Map<string, string>;
                    preferUnplugged: boolean | null;
                    type: string | null;
                };
                misc: {
                    extractHint: boolean;
                    hasBindingGyp: boolean;
                };
            }>;
        };
    } | undefined>;
    transformPnpSettings(pnpSettings: PnpSettings): Promise<void>;
    private isEsmEnabled;
    finalizeInstallWithPnp(pnpSettings: PnpSettings): Promise<void>;
    private locateNodeModules;
    private readonly unpluggedPaths;
    private unplugPackageIfNeeded;
    private shouldBeUnplugged;
    private unplugPackage;
    private getPackageInformation;
    private getDiskInformation;
}
